package com.piece_framework.piece_ide.form_designer.ui;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.IPartSelectionListener;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.piece_framework.piece_ide.form_designer.model.Field;

public abstract class AbstractFieldDetailsSection implements IPartSelectionListener {
    private IManagedForm fManagedForm;
    private SectionPart fSectionPart;
    private Field fField;

    public final void createContents(
            final Section parent, 
            final IManagedForm managedForm) {
        if (parent == null) {
            throw new NullPointerException();
        }
        if (managedForm == null) {
            throw new NullPointerException();
        }

        fManagedForm = managedForm;
        fSectionPart = new SectionPart(parent);
        parent.setClient(createDetailsComposite(parent));
    }

    public void selectionChanged(IFormPart part, ISelection selection) {
        if (fSectionPart == part) {
            return;
        }
        if (!(selection instanceof IStructuredSelection)) {
            return;
        }
        if (!(((IStructuredSelection) selection).getFirstElement()
                    instanceof Field)) {
            return;
        }

        Field field = (Field) ((IStructuredSelection) selection)
                                                    .getFirstElement();
        selectionFieldChanged(field);
        fField = field; 
    }

    public final IManagedForm getManagedForm() {
        return fManagedForm;
    }

    public final SectionPart getSectionPart() {
        return fSectionPart;
    }

    public final Field getField() {
        return fField;
    }

    protected Text createLabelAndText(
                        final Composite parent, 
                        final String label) {
        FormToolkit toolkit = getManagedForm().getToolkit();

        toolkit.createLabel(parent, label);

        Text text = toolkit.createText(parent, "");

        GridData layoutData = new GridData();
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.horizontalAlignment = GridData.FILL;
        text.setLayoutData(layoutData);
        
        return text; 
    }

    protected Button[] createLabelAndRadioButton(
                                final Composite parent, 
                                final String label) {
        FormToolkit toolkit = getManagedForm().getToolkit();

        toolkit.createLabel(parent, label);

        Composite composite = toolkit.createComposite(parent);
        composite.setLayout(new RowLayout(SWT.HORIZONTAL));
        Button yesButton = toolkit.createButton(composite, "͂", SWT.RADIO);
        Button noButton = toolkit.createButton(composite, "", SWT.RADIO);

        return new Button[] {yesButton, noButton};
    }

    protected abstract Composite createDetailsComposite(final Section parent);

    protected abstract void selectionFieldChanged(final Field field);
}
