package com.piece_framework.piece_ide.form_designer.ui;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.piece_framework.piece_ide.form_designer.model.Field;
import com.piece_framework.piece_ide.form_designer.model.Validator;
import com.piece_framework.piece_ide.form_designer.model.defined.DefinedRule;
import com.piece_framework.piece_ide.form_designer.model.defined.DefinedValidator;

public class FieldDetailsValidatorSection extends AbstractFieldDetailsSection {
    private class ValidatorUI {
        private DefinedValidator fDefinedValidator;
        private Validator fValidator;

        ValidatorUI(DefinedValidator definedValidator) {
            fDefinedValidator = definedValidator;
            fValidator = fDefinedValidator.create();
        }

        ValidatorUI(Validator validator) {
            fValidator = validator;
            fDefinedValidator = DefinedValidator.getDefinedValidator(
                                    fValidator);
        }

        public DefinedValidator getDefinedValidator() {
            return fDefinedValidator;
        }

        public Validator getValidator() {
            return fValidator;
        }
    }

    private Composite fValidatorDetails;
    private TableViewer fViewer;
    private Text fMessageText;
    private Map<DefinedRule, Widget[]> fRuleWidgets; 
    private ValidatorUI fCurrentSelection;

    @Override
    protected Composite createDetailsComposite(Section parent) {
        parent.setText("of[^");

        GridData layoutData = new GridData();
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.horizontalAlignment = GridData.FILL;
        parent.setLayoutData(layoutData);

        FormToolkit toolkit = getManagedForm().getToolkit();

        final Composite composite = toolkit.createComposite(parent);
        composite.setLayout(new GridLayout(2, false));

        layoutData = new GridData();
        layoutData.horizontalAlignment = GridData.FILL;
        layoutData.verticalAlignment = GridData.FILL;
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.grabExcessVerticalSpace = true;
        composite.setLayoutData(layoutData);

        createTable(composite);
        createButtons(composite);

        return composite;
    }

    private void createTable(final Composite composite) {
        final Table validatorTable = 
                        getManagedForm().getToolkit().createTable(
                                composite, SWT.SINGLE | SWT.FULL_SELECTION);
        validatorTable.setHeaderVisible(true);
        validatorTable.setLinesVisible(true);

        GridData layoutData = new GridData();
        layoutData.horizontalAlignment = GridData.FILL;
        layoutData.verticalAlignment = GridData.FILL;
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.grabExcessVerticalSpace = true;
        layoutData.minimumWidth = SWT.DEFAULT;
        validatorTable.setLayoutData(layoutData);

        createTableColumn(validatorTable, "of[^", 100);
        createTableColumn(validatorTable, "", 100);

        fViewer = new TableViewer(validatorTable);
        fViewer.setContentProvider(new ArrayContentProvider());
        fViewer.setLabelProvider(new ITableLabelProvider() {
            public Image getColumnImage(Object element, int columnIndex) {
                return null;
            }

            public String getColumnText(Object element, int columnIndex) {
                assert (element instanceof ValidatorUI);

                if (columnIndex == 0) {
                    return ((ValidatorUI) element)
                            .getDefinedValidator().getName();
                } else if (columnIndex == 1) {
                    return ((ValidatorUI) element)
                            .getDefinedValidator().getDescirption();
                }
                return null;
            }

            public void addListener(ILabelProviderListener listener) {
            }

            public void dispose() {
            }

            public boolean isLabelProperty(Object element, String property) {
                return false;
            }

            public void removeListener(ILabelProviderListener listener) {
            }
        });

        fViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                assert (((IStructuredSelection) event.getSelection())
                            .getFirstElement() instanceof ValidatorUI);

                ValidatorUI validatorUI = 
                    (ValidatorUI) ((IStructuredSelection) event.getSelection())
                                        .getFirstElement();
                if (fCurrentSelection == validatorUI || validatorUI == null) {
                    return;
                }
                fCurrentSelection = validatorUI;

                createValidatorDetailsWidget(composite, validatorUI);

                setValidatorValue(validatorUI);
            }
        });
    }

    private void createValidatorDetailsWidget(
                            final Composite composite, 
                            final ValidatorUI validatorUI) {
        FocusListener focusListener = new FocusListener() {
            public void focusGained(FocusEvent e) {
            }

            public void focusLost(FocusEvent event) {
                Validator validator = 
                    ((ValidatorUI) ((IStructuredSelection) fViewer
                            .getSelection()).getFirstElement()).getValidator();

                if (fMessageText == event.widget) {
                    validator.setMessage(fMessageText.getText());
                } else {
                    Iterator<DefinedRule> iterator = 
                        fRuleWidgets.keySet().iterator();
                    while (iterator.hasNext()) {
                        DefinedRule definedRule = iterator.next();
                        Widget[] widgets = fRuleWidgets.get(definedRule);

                        if (definedRule.getWidgetType() 
                                == DefinedRule.WidgetType.Text
                            && widgets[0] == event.widget) {
                            validator.setRule(
                                    definedRule.getName(), 
                                    ((Text) widgets[0]).getText());

                        } else if (definedRule.getWidgetType() 
                                        == DefinedRule.WidgetType.YesNo
                                   && (widgets[0] == event.widget 
                                       || widgets[1] == event.widget)) {
                            if (((Button) widgets[0]).getSelection()) {
                                validator.setRule(
                                        definedRule.getName(), "true");
                            } else {
                                validator.setRule(
                                        definedRule.getName(), "false");
                            }

                        } else {
                            assert (false);
                        }
                    }
                }

                getManagedForm().fireSelectionChanged(
                            getSectionPart(), 
                            new StructuredSelection(getField()));
            }
        };

        FormToolkit toolkit = getManagedForm().getToolkit();

        if (fValidatorDetails != null) {
            for (Control control : fValidatorDetails.getChildren()) {
                control.dispose();
            }
        } else {
            fValidatorDetails = toolkit.createComposite(composite);
            fValidatorDetails.setLayout(new GridLayout(2, false));

            GridData layoutData = new GridData();
            layoutData.horizontalSpan = 2;
            layoutData.grabExcessHorizontalSpace = true;
            layoutData.grabExcessVerticalSpace = true;
            layoutData.horizontalAlignment = GridData.FILL;
            layoutData.verticalAlignment = 
                    GridData.HORIZONTAL_ALIGN_BEGINNING;
            fValidatorDetails.setLayoutData(layoutData);
        }

        Label nameLabel = toolkit.createLabel(
                            fValidatorDetails, 
                            validatorUI.getDefinedValidator().getDisplayName());
        GridData layoutData = new GridData();
        layoutData.horizontalSpan = 2;
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.horizontalAlignment = GridData.FILL;
        nameLabel.setLayoutData(layoutData);

        fMessageText = createLabelAndText(fValidatorDetails, "bZ[W");
        fMessageText.addFocusListener(focusListener);

        fRuleWidgets = new HashMap<DefinedRule, Widget[]>();
        for (DefinedRule definedRule
                : validatorUI.getDefinedValidator().getRules()) {
            createRuleWidget(
                    fValidatorDetails, 
                    definedRule, 
                    focusListener);
        }

        layoutData = (GridData) fViewer.getTable().getLayoutData();
        layoutData.minimumWidth = fViewer.getTable().getSize().x
                                  - fViewer.getTable().getLocation().x
                                  + 5;
        composite.getParent().pack();
        layoutData.minimumWidth = 0;
    }

    private void setValidatorValue(ValidatorUI validatorUI) {
        Validator validator = validatorUI.getValidator();
        fMessageText.setText(validator.getMessage());
        for (DefinedRule definedRule
                    : validatorUI.getDefinedValidator().getRules()) {
            String value = validator.getRules().get(
                                    definedRule.getName());
            Widget[] widgets = fRuleWidgets.get(definedRule);
            if (definedRule.getWidgetType() 
                    == DefinedRule.WidgetType.Text) {
                ((Text) widgets[0]).setText(value);
            } else if (definedRule.getWidgetType() 
                    == DefinedRule.WidgetType.YesNo) {
                if (value.equals("true")) {
                    ((Button) widgets[0]).setSelection(true);
                } else {
                    ((Button) widgets[1]).setSelection(true);
                }
            }
        }
    }

    private TableColumn createTableColumn(final Table table, final String text, final int width) {
        TableColumn column = new TableColumn(table, SWT.NULL);
        column.setText(text);
        column.setWidth(width);

        return column;
    }

    private void createButtons(Composite composite) {
        final Composite buttons = 
            getManagedForm().getToolkit().createComposite(composite);
        GridData layoutData = new GridData();
        layoutData.verticalAlignment = GridData.FILL;
        buttons.setLayoutData(layoutData);

        buttons.setLayout(new GridLayout());

        Button addButton = createButton(buttons, "ǉ...");
        addButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                ValidatorSelectionDialog dialog = 
                            new ValidatorSelectionDialog(buttons.getShell());
                dialog.open();

                if (dialog.getReturnCode() == ValidatorSelectionDialog.CANCEL) {
                    return;
                }

                assert (dialog.getFirstResult() instanceof DefinedValidator);
                DefinedValidator definedValidator = 
                    (DefinedValidator) dialog.getFirstResult();

                Field field = getField();
                for (Validator validator : getField().getValidators()) {
                    if (validator.getName().equals(
                            definedValidator.getName())) {
                        MessageBox messageBox = new MessageBox(
                                FieldDetailsValidatorSection.this
                                    .getSectionPart().getSection().getShell(), 
                                SWT.ICON_INFORMATION);
                        messageBox.setText("");
                        messageBox.setMessage(
                                validator.getName() + "of[V͊ɑ݂܂B");
                        messageBox.open();
                        return;
                    }
                }

                ValidatorUI validatorUI = new ValidatorUI(definedValidator);
                field.addValidator(validatorUI.getValidator());

                fViewer.add(validatorUI);
                fViewer.setSelection(new StructuredSelection(validatorUI));
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button delButton = createButton(buttons, "폜...");
        delButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                ValidatorUI validatorUI = 
                    (ValidatorUI) ((IStructuredSelection) 
                            fViewer.getSelection()).getFirstElement();

                getField().removeValidator(validatorUI.getValidator());
                fViewer.remove(validatorUI);
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button upButton = createButton(buttons, "");
        upButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                int index = fViewer.getTable().getSelectionIndex();
                if (index == 0) {
                    return;
                }

                ValidatorUI tmpValidatorUI = 
                    (ValidatorUI) fViewer.getElementAt(index - 1);
                fViewer.replace(fViewer.getElementAt(index), index - 1);
                fViewer.replace(tmpValidatorUI, index);

                fViewer.setSelection(
                        new StructuredSelection(
                                (ValidatorUI) fViewer.getElementAt(index - 1)));
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button downButton = createButton(buttons, "");
        downButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                int index = fViewer.getTable().getSelectionIndex();
                if (index + 1 == fViewer.getTable().getItemCount()) {
                    return;
                }

                ValidatorUI tmpValidatorUI = 
                    (ValidatorUI) fViewer.getElementAt(index + 1);
                fViewer.replace(fViewer.getElementAt(index), index + 1);
                fViewer.replace(tmpValidatorUI, index);

                fViewer.setSelection(
                        new StructuredSelection(
                                (ValidatorUI) fViewer.getElementAt(index + 1)));
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });
    }

    private Button createButton(final Composite parent, final String label) {
        Button button = getManagedForm().getToolkit().createButton(
                                parent, 
                                label, 
                                SWT.PUSH);

        GridData layoutData = new GridData();
        layoutData.horizontalAlignment = GridData.FILL;
        button.setLayoutData(layoutData);

        return button;
    }

    private void createRuleWidget(
                    final Composite parent, 
                    final DefinedRule definedRule, 
                    final FocusListener focusListener) {
        if (definedRule.getWidgetType() == DefinedRule.WidgetType.Text) {
            Text text = createLabelAndText(parent, definedRule.getName());
            text.addFocusListener(focusListener);
            fRuleWidgets.put(
                    definedRule, 
                    new Widget[]{text});
        } else if (definedRule.getWidgetType()
                        == DefinedRule.WidgetType.YesNo) {
            Button[] radioButtons = createLabelAndRadioButton(
                                        parent, 
                                        definedRule.getName());
            for (Button radioButton : radioButtons) {
                radioButton.addFocusListener(focusListener);
            }
            fRuleWidgets.put(
                    definedRule, 
                    new Widget[]{radioButtons[0], radioButtons[1]});
        } else {
            assert (false);
        }
    }

    @Override
    protected void selectionFieldChanged(Field field) {
        fViewer.getTable().removeAll();
        fViewer.refresh();
        for (Validator validator : field.getValidators()) {
            fViewer.add(new ValidatorUI(validator));
        }
    }
}
