// $Id: FieldsMasterSectionPart.java 39 2008-03-12 17:04:28Z matsu $
package com.piece_framework.piece_ide.form_designer.ui;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.IPartSelectionListener;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.ho.yaml.Yaml;

import com.piece_framework.piece_ide.form_designer.model.Field;
import com.piece_framework.piece_ide.form_designer.model.Validator;

public class FieldsMasterSectionPart extends SectionPart implements IPartSelectionListener {
    private TableViewer fViewer;
    private Field fCurrentSelection;

    public FieldsMasterSectionPart(Composite parent, IManagedForm managedForm) {
        super(parent, 
              managedForm.getToolkit(), 
              Section.TITLE_BAR | Section.DESCRIPTION);
        initialize(managedForm);

        createContents(getSection(), managedForm.getToolkit());

        getManagedForm().addPart(this);
    }

    private void createContents(Section section, FormToolkit toolkit) {
        section.setText("tB[hꗗ");
        section.setDescription("ҏWtB[hIĂB");

        Composite composite = toolkit.createComposite(section);
        composite.setLayout(new GridLayout(2, false));

        createTable(composite);
        createButtons(composite);

        section.setClient(composite);
    }

    private void createTable(Composite composite) {
        Table table = getManagedForm().getToolkit().createTable(
                        composite, 
                        SWT.SINGLE | SWT.FULL_SELECTION);
        table.setHeaderVisible(true);
        table.setLinesVisible(true);

        GridData layoutData = new GridData();
        layoutData.horizontalAlignment = GridData.FILL;
        layoutData.verticalAlignment = GridData.FILL;
        layoutData.grabExcessHorizontalSpace = true;
        layoutData.grabExcessVerticalSpace = true;
        table.setLayoutData(layoutData);
       
        createTableColumn(table, "tB[h", 100);
        createTableColumn(table, "", 100);

        fViewer = new TableViewer(table);
        fViewer.setContentProvider(new ArrayContentProvider());
        fViewer.setLabelProvider(new ITableLabelProvider() {
            public Image getColumnImage(Object element, int columnIndex) {
                return null;
            }

            public String getColumnText(Object element, int columnIndex) {
                if (element == null) {
                    return "";
                }
                if (!(element instanceof Field)) {
                    return "";
                }
                
                Field field = (Field) element;
                if (columnIndex == 0) {
                    return field.getName();
                } else if (columnIndex == 1) {
                    return field.getDescription();
                }
                return "";
            }

            public void addListener(ILabelProviderListener listener) {
            }

            public void dispose() {
            }

            public boolean isLabelProperty(Object element, String property) {
                return false;
            }

            public void removeListener(ILabelProviderListener listener) {
            }
        });

        fViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                Field field = (Field) ((IStructuredSelection)
                        event.getSelection()).getFirstElement();
                if (fCurrentSelection == field) {
                    return;
                }
                fCurrentSelection = field;
                
                FieldsMasterSectionPart.this.getManagedForm()
                    .fireSelectionChanged(
                                FieldsMasterSectionPart.this,  
                                event.getSelection());
            }
        });

        setFieldTableFromYAMLFile();
    }

    private void setFieldTableFromYAMLFile() {
        assert (!(getManagedForm().getInput() instanceof IFileEditorInput));

        BufferedInputStream bufferedIn = null;
        ByteArrayOutputStream byteOut = null;
        try {
            bufferedIn = new BufferedInputStream(
                    ((IFileEditorInput) getManagedForm().getInput()).getFile()
                        .getContents(true));
            byteOut = new ByteArrayOutputStream();
            int read = 0;
            while ((read = bufferedIn.read()) != -1) {
                byteOut.write(read);
            }

            List<Map<String, Object>> fieldList = (List<Map<String, Object>>) Yaml.load(byteOut.toString("UTF-8"));
            for (Map<String, Object> fieldMap : fieldList) {
                fViewer.add(getField(fieldMap));
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (CoreException e) {
            e.printStackTrace();
        } finally {
            try {
                if (bufferedIn != null) {
                    bufferedIn.close();
                }
            } catch (IOException ioe) {
            }
            try {
                if (byteOut != null) {
                    byteOut.close();
                }
            } catch (IOException ioe) {
            }
        }
    }

    private Field getField(Map<String, Object> fieldMap) {
        Field field = new Field("");

        Iterator<String> iterator = fieldMap.keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            if ("name".equals(key)) {
                if ((String) fieldMap.get(key) != null) {
                    field.setName((String) fieldMap.get(key));
                }
            } else if ("description".equals(key)) {
                if ((String) fieldMap.get(key) != null) {
                    field.setDescription((String) fieldMap.get(key));
                }
            } else if ("required".equals(key)) {
                field.setRequired(true);
                if ((String) ((Map) fieldMap.get(key)).get("message") != null) {
                    field.setMessage((String) ((Map) fieldMap.get(key)).get("message"));
                }
            } else if ("validator".equals(key)) {
                List<Map<String, Object>> validatorList = (List<Map<String, Object>>) fieldMap.get(key);

                for (Map<String, Object> validatorMap : validatorList) {
                    field.addValidator(getValidator(validatorMap));
                }
            }
        }

        return field;
    }

    private Validator getValidator(Map<String, Object> validatorMap) {
        String name = "";
        String message = "";
        Map<String, String> ruleMap = new LinkedHashMap<String, String>();

        Iterator<String> validatorIterator = validatorMap.keySet().iterator();
        while (validatorIterator.hasNext()) {
            String validatorKey = validatorIterator.next();
            if ("name".equals(validatorKey)) {
                if ((String) validatorMap.get(validatorKey) != null) {
                    name = (String) validatorMap.get(validatorKey); 
                }                                
            } else if ("message".equals(validatorKey)) {
                if ((String) validatorMap.get(validatorKey) != null) {
                    message = (String) validatorMap.get(validatorKey); 
                }
            } else if ("rule".equals(validatorKey)) {
                Map<String, Object> srcRuleMap = (Map<String, Object>) validatorMap.get(validatorKey);
                Iterator<String> ruleIterator = srcRuleMap.keySet().iterator();
                while (ruleIterator.hasNext()) {
                    String ruleKey = ruleIterator.next();
                    if (srcRuleMap.get(ruleKey) == null) {
                        ruleMap.put(ruleKey, "");
                    } else if (srcRuleMap.get(ruleKey) instanceof String) {
                        ruleMap.put(ruleKey, (String) srcRuleMap.get(ruleKey));
                    } else if (srcRuleMap.get(ruleKey) instanceof Boolean) {
                        ruleMap.put(ruleKey, ((Boolean) srcRuleMap.get(ruleKey)).toString());
                    }
                }
            }
        }

        Validator validator = new Validator(name, ruleMap);
        validator.setMessage(message);

        return validator;
    }

    private void createButtons(Composite composite) {
        final Composite buttons = 
            getManagedForm().getToolkit().createComposite(composite);
        GridData layoutData = new GridData();
        layoutData.verticalAlignment = GridData.FILL;
        buttons.setLayoutData(layoutData);

        buttons.setLayout(new GridLayout());

        Button addButton = createButton(buttons, "ǉ(&A)...");
        addButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                InputDialog dialog = new InputDialog(
                                        buttons.getShell(), 
                                        "tB[h", 
                                        "tB[h͂ĂB", 
                                        null, 
                                        null);
                dialog.open();
                if (dialog.getValue() == null || dialog.getValue().equals("")) {
                    return;
                }

                Field field = new Field(dialog.getValue());
                fViewer.add(field);
                fViewer.setSelection(new StructuredSelection(field));
                markDirty();
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button delButton = createButton(buttons, "폜(&D)...");
        delButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                Field field = 
                    (Field) ((IStructuredSelection) fViewer.getSelection())
                    .getFirstElement();
                fViewer.remove(field);
                markDirty();
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button upButton = createButton(buttons, "");
        upButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                int index = fViewer.getTable().getSelectionIndex();
                if (index == 0) {
                    return;
                }

                Field tmpField = (Field) fViewer.getElementAt(index - 1);
                fViewer.replace(fViewer.getElementAt(index), index - 1);
                fViewer.replace(tmpField, index);

                fViewer.setSelection(
                        new StructuredSelection(
                                (Field) fViewer.getElementAt(index - 1)));
                markDirty();
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        Button downButton = createButton(buttons, "");
        downButton.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                int index = fViewer.getTable().getSelectionIndex();
                if (index + 1 == fViewer.getTable().getItemCount()) {
                    return;
                }

                Field tmpField = (Field) fViewer.getElementAt(index + 1);
                fViewer.replace(fViewer.getElementAt(index), index + 1);
                fViewer.replace(tmpField, index);

                fViewer.setSelection(
                        new StructuredSelection(
                                (Field) fViewer.getElementAt(index + 1)));
                markDirty();
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });
    }

    private TableColumn createTableColumn(
                            final Table table, 
                            final String text, 
                            final int width) {
        TableColumn column = new TableColumn(table, SWT.NULL);
        column.setText(text);
        column.setWidth(width);

        return column;
    }

    private Button createButton(final Composite parent, final String label) {
        Button button = getManagedForm().getToolkit().createButton(
                                parent, 
                                label, 
                                SWT.PUSH);

        GridData layoutData = new GridData();
        layoutData.horizontalAlignment = GridData.FILL;
        button.setLayoutData(layoutData);

        return button;
    }

    public void selectionChanged(IFormPart part, ISelection selection) {
        if (part == this) {
            return;
        }
        if (!(selection instanceof IStructuredSelection)) {
            return;
        }

        fViewer.refresh(
                (Field) ((IStructuredSelection) selection).getFirstElement());
        markDirty();
    }

    @Override
    public void commit(boolean onSave) {
        super.commit(onSave);

        if (!onSave) {
            return;
        }

        assert (!(getManagedForm().getInput() instanceof IFileEditorInput));

        try {
            ((IFileEditorInput) getManagedForm().getInput()).getFile().setContents(
                    new ByteArrayInputStream(
                            formatYAMLString(getYAML()).getBytes("UTF-8")),  //$NON-NLS-1$
                    true,
                    false,
                    null);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    private String getYAML() {
        List<Map<String, Object>> fieldList = new ArrayList<Map<String, Object>>();
        for (int i = 0, n = fViewer.getTable().getItemCount(); i < n; i++) {
            Field field = (Field) fViewer.getElementAt(i);

            Map<String, Object> fieldMap = getYAMLForField(field);

            List<Map<String, Object>> validatorList = new ArrayList<Map<String, Object>>();
            for (Validator validator : field.getValidators()) {
                validatorList.add(getYAMLForValidator(validator));
            }
            if (validatorList.size() > 0) {
                fieldMap.put("validator", validatorList);
            }

            fieldList.add(fieldMap);
        }

        return Yaml.dump(fieldList);
    }

    private Map<String, Object> getYAMLForValidator(Validator validator) {
        Map<String, Object> validatorMap = new LinkedHashMap<String, Object>();
        validatorMap.put("name", validator.getName());
        
        Map<String, String> ruleMap = new LinkedHashMap<String, String>();
        Iterator<String> iterator = validator.getRules().keySet().iterator();
        while (iterator.hasNext()) {
            String key = iterator.next();
            ruleMap.put(key, validator.getRules().get(key));
        }
        validatorMap.put("rule", ruleMap);

        validatorMap.put("message", validator.getMessage());
        return validatorMap;
    }

    private Map<String, Object> getYAMLForField(Field field) {
        Map<String, Object> fieldMap = new LinkedHashMap<String, Object>();

        fieldMap.put("name", field.getName());
        fieldMap.put("description", field.getDescription());
        if (field.isRequired()) {
            Map<String, String> messageMap = new LinkedHashMap<String, String>();
            messageMap.put("message", field.getMessage());
            fieldMap.put("required", messageMap);
        }
        return fieldMap;
    }

    private String formatYAMLString(String yamlString) {
        String result = yamlString;

        result = result.replace("\r\n", "\n"); //$NON-NLS-1$ //$NON-NLS-2$
        result = result.replace("--- \n", ""); //$NON-NLS-1$ //$NON-NLS-2$
        result = result.replace("\"", ""); //$NON-NLS-1$ //$NON-NLS-2$
        result = result.replace(
                " !java.util.LinkedHashMap", ""); //$NON-NLS-1$ //$NON-NLS-2$
        result = result.replaceAll("-\n *", "- "); //$NON-NLS-1$ //$NON-NLS-2$
        result = result.replace(": \n", ":\n"); //$NON-NLS-1$ //$NON-NLS-2$
        if (result.length() > 0) {
            result = result.substring(0, result.length() - 1);
        }
        if (!result.endsWith("\n")) {
            result += "\n";
        }

        return result;
    }

}
